/*
 * Kosmos.
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package hu.midori.kosmos.server.jira;

import hu.midori.kosmos.model.ZoomableImageLocator;
import hu.midori.kosmos.server.AbstractKosmosService;
import hu.midori.kosmos.server.util.ChartUtils;
import hu.midori.kosmos.server.util.ColorCodedKey;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * Encapsulates the common functionality of concrete
 * JIRA service implemnetations.
 *
 * @author <a href="mailto:aron.gombas@midori.hu">Aron Gombas</a>
 * @version $Id$
 */
public abstract class AbstractJiraServiceImpl extends AbstractKosmosService {
    /** Generates issues-per-status chart. */
    protected ZoomableImageLocator generateIssuePerStatusChart(String prefix, int openIssues, int codingInProgressIssues, int reopenedIssues, int resolvedIssues, int closedIssues) {
        List<Map.Entry<ColorCodedKey, Integer>> issuesPerStatusMap = new ArrayList<Map.Entry<ColorCodedKey, Integer>>();
        if(openIssues > 0)
            issuesPerStatusMap.addAll(Collections.singletonMap(new ColorCodedKey(getResourceString("hu.midori.kosmos.server.jira.jira_monitoring", "jiraproject.openIssues"), JiraConstants.OPEN_COLOR), openIssues).entrySet());
        if(codingInProgressIssues > 0)
            issuesPerStatusMap.addAll(Collections.singletonMap(new ColorCodedKey(getResourceString("hu.midori.kosmos.server.jira.jira_monitoring", "jiraproject.codingInProgressIssues"), JiraConstants.CODING_IN_PROGRESS_COLOR), codingInProgressIssues).entrySet());
        if(reopenedIssues > 0)
            issuesPerStatusMap.addAll(Collections.singletonMap(new ColorCodedKey(getResourceString("hu.midori.kosmos.server.jira.jira_monitoring", "jiraproject.reopenedIssues"), JiraConstants.REOPENED_COLOR), reopenedIssues).entrySet());
        if(resolvedIssues > 0)
            issuesPerStatusMap.addAll(Collections.singletonMap(new ColorCodedKey(getResourceString("hu.midori.kosmos.server.jira.jira_monitoring", "jiraproject.resolvedIssues"), JiraConstants.RESOLVED_COLOR), resolvedIssues).entrySet());
        if(closedIssues > 0)
            issuesPerStatusMap.addAll(Collections.singletonMap(new ColorCodedKey(getResourceString("hu.midori.kosmos.server.jira.jira_monitoring", "jiraproject.closedIssues"), JiraConstants.CLOSED_COLOR), closedIssues).entrySet());

        return storeChart(prefix + "_issues_per_status", ChartUtils.generateColorCodedPieChart(ChartUtils.collectionToPieDataset(issuesPerStatusMap)));
    }

    /** Generates issues-per-priority chart. */
    protected ZoomableImageLocator generateIssuePerPriorityChart(String prefix, int blockerOpenIssues, int criticalOpenIssues, int majorOpenIssues, int minorOpenIssues, int trivialOpenIssues, int optionalOpenIssues) {
        List<Map.Entry<ColorCodedKey, Integer>> openIssuesPerPriorityMap = new ArrayList<Map.Entry<ColorCodedKey, Integer>>();
        if(blockerOpenIssues > 0)
            openIssuesPerPriorityMap.addAll(Collections.singletonMap(new ColorCodedKey(getResourceString("hu.midori.kosmos.server.jira.jira_monitoring", "jiraproject.blockerOpenIssues"), JiraConstants.BLOCKER_COLOR), blockerOpenIssues).entrySet());
        if(criticalOpenIssues > 0)
            openIssuesPerPriorityMap.addAll(Collections.singletonMap(new ColorCodedKey(getResourceString("hu.midori.kosmos.server.jira.jira_monitoring", "jiraproject.criticalOpenIssues"), JiraConstants.CRITICAL_COLOR), criticalOpenIssues).entrySet());
        if(majorOpenIssues > 0)
            openIssuesPerPriorityMap.addAll(Collections.singletonMap(new ColorCodedKey(getResourceString("hu.midori.kosmos.server.jira.jira_monitoring", "jiraproject.majorOpenIssues"), JiraConstants.MAJOR_COLOR), majorOpenIssues).entrySet());
        if(minorOpenIssues > 0)
            openIssuesPerPriorityMap.addAll(Collections.singletonMap(new ColorCodedKey(getResourceString("hu.midori.kosmos.server.jira.jira_monitoring", "jiraproject.minorOpenIssues"), JiraConstants.MINOR_COLOR), minorOpenIssues).entrySet());
        if(trivialOpenIssues > 0)
            openIssuesPerPriorityMap.addAll(Collections.singletonMap(new ColorCodedKey(getResourceString("hu.midori.kosmos.server.jira.jira_monitoring", "jiraproject.trivialOpenIssues"), JiraConstants.TRIVIAL_COLOR), trivialOpenIssues).entrySet());
        if(optionalOpenIssues > 0)
            openIssuesPerPriorityMap.addAll(Collections.singletonMap(new ColorCodedKey(getResourceString("hu.midori.kosmos.server.jira.jira_monitoring", "jiraproject.optionalOpenIssues"), JiraConstants.OPTIONAL_COLOR), optionalOpenIssues).entrySet());

        return storeChart(prefix + "_open_issues_per_priority", ChartUtils.generateColorCodedPieChart(ChartUtils.collectionToPieDataset(openIssuesPerPriorityMap)));
    }

    /** Generates open-issues-per-assignee chart. */
    protected ZoomableImageLocator generateOpenIssuesPerAssigneeChart(String prefix, List<Map.Entry<String, Integer>> openIssuesPerAssignee) {
        return storeChart(prefix + "_open_issues_per_assignee", ChartUtils.generatePieChart(ChartUtils.collectionToPieDataset(openIssuesPerAssignee, ChartUtils.MAX_ITEMS)));
    }
}
