/*
 * Kosmos.
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package hu.midori.kosmos.server.svn;

import hu.midori.kosmos.model.SvnRepository;
import hu.midori.kosmos.protocol.SvnService;
import hu.midori.kosmos.server.AbstractKosmosServiceTest;

import java.util.List;

import org.apache.commons.lang.StringUtils;

/**
 * Unit test.
 *
 * @author <a href="mailto:aron.gombas@midori.hu">Aron Gombas</a>
 * @version $Id$
 */
public class SvnServiceTest extends AbstractKosmosServiceTest {
    private SvnService service = new SvnServiceImpl();

    /** Tests with invalid URL. */
    public void testInvalidUrl() {
        assertEquals(0, service.getRepositories("http://www.nosuchdomain.com/svn").size());
    }

    /** Tests accessing secured repositories. */
    public void testAuthentication() {
        // test secured without signing in
        assertEquals(0, service.getRepositories("https://svn.labs.jboss.com/trunk/labs/kosmos").size());// TODO specify secured here

        // test secured with signing in
        testRepository("https://myusername:mypassword@svn.labs.jboss.com/trunk/labs/kosmos");// TODO
    }

    public void testApacheRepositories() {
        testRepository("http://svn.apache.org/repos/asf/commons");
        testRepository("http://svn.apache.org/repos/asf/db");
        testRepository("http://svn.apache.org/repos/asf/incubator/agila");
    }

    public void testJBossRepositories() {
        testRepository("http://anonsvn.labs.jboss.com/trunk/labs/demo");
        testRepository("http://anonsvn.labs.jboss.com/trunk/labs/jbossprofiler");
        testRepository("http://anonsvn.labs.jboss.com/trunk/labs/jbossweb");
        testRepository("http://anonsvn.labs.jboss.com/trunk/labs/jrunit");
        testRepository("http://anonsvn.labs.jboss.com/trunk/labs/kosmos");
        testRepository("http://anonsvn.labs.jboss.com/trunk/labs/portletswap");
        testRepository("http://anonsvn.labs.jboss.com/trunk/labs/reportingservices");
    }

    // TODO add more repositories here

    /** Tests a public instance. */
    protected void testRepository(String url) {
        List<SvnRepository> repositories = service.getRepositories(url);
        assertTrue(repositories.size() > 0);

        // test repositories
        for(SvnRepository repository : repositories) {
            assertFalse(StringUtils.isBlank(repository.getLocation()));
            assertTrue(repository.getRevision() > 0);
            assertTrue(repository.getCreatedDate().getTime() > 0);
            // TODO assert getCommits()
            assertTrue(repository.getCommitsTotal() >= 0);
            assertTrue((repository.getCommitsToday() >= 0) && (repository.getCommitsToday() <= repository.getCommitsTotal()));
            assertTrue((repository.getCommitsLast7Days() >= 0) && (repository.getCommitsLast7Days() <= repository.getCommitsTotal()));
            assertTrue((repository.getCommitsLast31Days() >= 0) && (repository.getCommitsLast31Days() <= repository.getCommitsTotal()));
            assertTrue(repository.getCommittersTotal() >= 0);
            assertTrue((repository.getCommittersToday() >= 0) && (repository.getCommittersToday() <= repository.getCommittersTotal()));
            assertTrue((repository.getCommittersLast7Days() >= 0) && (repository.getCommittersLast7Days() <= repository.getCommittersTotal()));
            assertTrue((repository.getCommittersLast31Days() >= 0) && (repository.getCommittersLast31Days() <= repository.getCommittersTotal()));
            assertFalse(StringUtils.isBlank(repository.getCommitsPerAuthorChartLocator().getLoresUrl()));
            assertFalse(StringUtils.isBlank(repository.getCommitsPerFileChartLocator().getLoresUrl()));
            assertFalse(StringUtils.isBlank(repository.getCommitsPerWeekChartLocator().getLoresUrl()));
            assertFalse(StringUtils.isBlank(repository.getRepoEntriesPerWeekChartLocator().getLoresUrl()));
            assertTrue(repository.getDirs() >= 0);
            assertTrue(repository.getFiles() >= 0);
            assertTrue(repository.getTotalFileSize() >= 0);
            // TODO assert getCommitsPerAuthor()
            // TODO assert getCommitsPerFile()
            assertTrue(repository.getAge() >= 0);
            assertTrue((repository.getLatestTouchAge() >= 0) && (repository.getLatestTouchAge() < repository.getAge()));
            assertTrue(repository.getLatestTouchAgeInDays() >= 0);
            assertTrue((repository.getActivity() >= 0) && (repository.getActivity() <= 1.0));
        }
    }
}
